import java.util.*;
import java.io.*;

public class ESMReader {

  private static final int[] pow = calcpow();   // Tableau des puissances de deux
  
  private static FileInputStream br = null;     // Lecteur de fichier
  
  private static String datalabel = null;
  private static int datasize = 0;
  private static String databody = null;
  

  // Traitement principal
  public static void main(String[] args) {
    try {
  		br = new FileInputStream(args[0]);    // Ouverture du fichier pass en argument
  		boolean bookok = false;

      // Cration des rpertoires
      (new File("C:\\Parchemins")).mkdirs();
      (new File("C:\\Livres")).mkdirs();
      
      while (true) {
        if (bookok == false) {            // Si on n'est pas dj sur un BOOK, on cherche le suivant
          boolean eof = find("BOOK");     // Cherche le mot BOOK dans le fichier
          if (eof == true) break;         // Fin du fichier ? => on arrte

          skip(12);                       // On passe les 12 octets inutiles aprs le mot BOOK
        } else {
          skip(8);                        // On est dj sur un mot BOOK, il reste 8 octets  sauter
        }
          
    
        readData("NAME");                 // On lit le bloc NAME
        String name = databody.trim();    // Nom interne (ID) du book
        
        readData("MODL");                 // On lit le bloc MODL
        String modl = databody.trim();
        
        readData("FNAM");                 // On lit le bloc FNAM
        String fnam = databody.trim();    // Nom affichable du book
        
        readData("BKDT");                 // On lit le bloc BKDT
        String bkdt = databody;

  		  boolean parchemin = (bkdt.charAt(8) == 1);    // Le 8me octet de BKDT indique s'il s'agit d'un parchemin
        
        // Trace
        System.out.println( (parchemin ? "Parchemin : " : "Livre : ") + fnam + " (" + name + ")");

        // On lit le bloc ITEX
        readData("ITEX");
        while (databody == null) {  // On passe toutes les sections (p.ex SCRI) jusqu' ITEX
          skip(datasize);
          readData("ITEX");
        }
        String itex = databody.trim();


        // Section TEXT si prsente (texte du livre)
        readData("TEXT");
        String text = "";

        if (datasize < 0) {
          System.out.println("!!! ERREUR - LIVRE IGNORE !!!");
        } else {  
          text = (databody != null) ? databody.trim() : "(Ce livre ne contient rien)";  // Traitement des books vides
        }
        
        // Enregistrement du fichier HTML contenant le texte du book
        save(parchemin, name + " = " + fnam, text);

        // Si la section TEXT est absente et qu'on est dj arriv sur le prochain BOOK, on enchaine directement
        bookok = (datalabel.equals("BOOK"));
    		  
//    		System.out.println("\n\n\n");
  		}
  
      br.close();       // Fin de la lecture du fichier
      br = null;
    } catch (Exception e) {
      e.printStackTrace();
    }

  }


  // Avance dans le fichier jusqu'au texte indiqu. Renvoie true si le fichier est fini
  private static boolean find(String s) throws Exception {
    // On commence par lire un bloc de n.length octets.
    // Si ce bloc n'est pas le texte cherch, on lui enlve le 1er octet, on lit l'octet suivant dans le fichier et on le rajoute au bloc
		byte[] data = new byte[s.length()];
		br.read(data);
		
		String sb = new String(data);
		
		data = new byte[1];
		while ( ! sb.equals(s) ) {
		  sb = sb.substring(1);
		  if (br.read(data) < 0) return true;
		  sb = sb + new String(data);
		}
		return false;
  }

  // Saute N octets
  private static void skip(int size) throws Exception  {
    for (int i=0; i<size; i++) br.read();
  }
  
  // Lit N octets
  private static String read(int size) throws Exception  {
		byte[] data = new byte[size];
		br.read(data);
		return new String(data);
  }
  
  // Lit le bloc 'label'. Si le bloc courant n'est pas 'label', le 'databody' est null.
  private static boolean readData(String label) throws Exception  {
    datalabel = read(label.length());
    datasize = getSize(4);
    if (datasize < 0) {
      databody = null;
      return false;
    }

    if ( ! datalabel.equals(label) ) {
      System.out.println("attendu=" + label + "  lu="+datalabel);
      databody = null;
      return false;
    }

if (label.equals("TEXT")) System.out.println("text size = " + datasize);

    databody = read(datasize);
    
//    System.out.println(label + "=" + res);
    return true;
  }
  
  // Calcule la taille du bloc
  private static int getSize(int lon) throws Exception  {
    int res = 0;
    for (int i=0; i<lon; i++) {
      int v = br.read();
      if (v > 256) {
        System.out.println("Octet lu incorrect : " + v);
        return -1;   // ERREUR !!
      }
      res += v * pow[i];
    }
    
    return res;
  }

  private static int val(char c) {
    return (int)((c>=0) ? c : 256+c);
  }

  // Enregistre le texte dans un fichier HTML  
  private static void save(boolean parchemin, String bookname, String booktext) throws Exception  {
    String fname = ( (parchemin == true) ? ("C:\\Parchemins\\" + bookname + ".html") : ("C:\\Livres\\" + bookname + ".html") );
		FileWriter writer = new FileWriter(fname);
		writer.write("<HTML><HEAD><TITLE>" + bookname + "</TITLE></HEAD><BODY>\r\n");
		writer.write(booktext);
		writer.write("\r\n</BODY></HTML>");
		writer.flush();
		writer.close();
	}

  // Calcule les 8 premires puissances de 2
  private static final int[] calcpow() {
    int[] res = new int[8];
    int r = 1;
    for (int i=0; i<8; i++) {
      res[i] = r;
      r = r * 256;
    }
    return res;
  }
}